@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 2.0.  */
+/* A Bison parser, made by GNU Bison 2.1.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 /* As a special exception, when this file is copied by Bison into a
    Bison output file, you may use that output file without restriction.
@@ -36,6 +36,9 @@
 /* Identify Bison output.  */
 #define YYBISON 1
 
+/* Bison version.  */
+#define YYBISON_VERSION "2.1"
+
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
 
@@ -83,6 +86,7 @@
      NUM = 285
    };
 #endif
+/* Tokens.  */
 #define K_OPTIONS 258
 #define K_DEVICE 259
 #define K_AUTOINIT 260
@@ -128,24 +132,19 @@
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: parser.y,v 1.16 2005/07/05 21:15:45 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -153,6 +152,7 @@
 #endif
 
 #include <stdio.h>
+#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
@@ -189,14 +189,19 @@ int yyerror(char *s);
 # define YYERROR_VERBOSE 0
 #endif
 
+/* Enabling the token table.  */
+#ifndef YYTOKEN_TABLE
+# define YYTOKEN_TABLE 0
+#endif
+
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 60 "parser.y"
+#line 56 "parser.y"
 typedef union YYSTYPE {
 	char *str;
 	long  num;
 } YYSTYPE;
-/* Line 190 of yacc.c.  */
-#line 200 "parser.c"
+/* Line 196 of yacc.c.  */
+#line 205 "parser.c"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -207,18 +212,37 @@ typedef union YYSTYPE {
 /* Copy the second part of user declarations.  */
 
 
-/* Line 213 of yacc.c.  */
-#line 212 "parser.c"
+/* Line 219 of yacc.c.  */
+#line 217 "parser.c"
+
+#if ! defined (YYSIZE_T) && defined (__SIZE_TYPE__)
+# define YYSIZE_T __SIZE_TYPE__
+#endif
+#if ! defined (YYSIZE_T) && defined (size_t)
+# define YYSIZE_T size_t
+#endif
+#if ! defined (YYSIZE_T) && (defined (__STDC__) || defined (__cplusplus))
+# include <stddef.h> /* INFRINGES ON USER NAME SPACE */
+# define YYSIZE_T size_t
+#endif
+#if ! defined (YYSIZE_T)
+# define YYSIZE_T unsigned int
+#endif
+
+#ifndef YY_
+# if YYENABLE_NLS
+#  if ENABLE_NLS
+#   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
+#   define YY_(msgid) dgettext ("bison-runtime", msgid)
+#  endif
+# endif
+# ifndef YY_
+#  define YY_(msgid) msgid
+# endif
+#endif
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
-# ifndef YYFREE
-#  define YYFREE free
-# endif
-# ifndef YYMALLOC
-#  define YYMALLOC malloc
-# endif
-
 /* The parser invokes alloca or malloc; define the necessary symbols.  */
 
 # ifdef YYSTACK_USE_ALLOCA
@@ -227,6 +251,10 @@ typedef union YYSTYPE {
 #    define YYSTACK_ALLOC __builtin_alloca
 #   else
 #    define YYSTACK_ALLOC alloca
+#    if defined (__STDC__) || defined (__cplusplus)
+#     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
+#     define YYINCLUDED_STDLIB_H
+#    endif
 #   endif
 #  endif
 # endif
@@ -234,13 +262,39 @@ typedef union YYSTYPE {
 # ifdef YYSTACK_ALLOC
    /* Pacify GCC's `empty if-body' warning. */
 #  define YYSTACK_FREE(Ptr) do { /* empty */; } while (0)
-# else
-#  if defined (__STDC__) || defined (__cplusplus)
-#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
-#   define YYSIZE_T size_t
+#  ifndef YYSTACK_ALLOC_MAXIMUM
+    /* The OS might guarantee only one guard page at the bottom of the stack,
+       and a page size can be as small as 4096 bytes.  So we cannot safely
+       invoke alloca (N) if N exceeds 4096.  Use a slightly smaller number
+       to allow for a few compiler-allocated temporary stack slots.  */
+#   define YYSTACK_ALLOC_MAXIMUM 4032 /* reasonable circa 2005 */
 #  endif
+# else
 #  define YYSTACK_ALLOC YYMALLOC
 #  define YYSTACK_FREE YYFREE
+#  ifndef YYSTACK_ALLOC_MAXIMUM
+#   define YYSTACK_ALLOC_MAXIMUM ((YYSIZE_T) -1)
+#  endif
+#  ifdef __cplusplus
+extern "C" {
+#  endif
+#  ifndef YYMALLOC
+#   define YYMALLOC malloc
+#   if (! defined (malloc) && ! defined (YYINCLUDED_STDLIB_H) \
+	&& (defined (__STDC__) || defined (__cplusplus)))
+void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
+#   endif
+#  endif
+#  ifndef YYFREE
+#   define YYFREE free
+#   if (! defined (free) && ! defined (YYINCLUDED_STDLIB_H) \
+	&& (defined (__STDC__) || defined (__cplusplus)))
+void free (void *); /* INFRINGES ON USER NAME SPACE */
+#   endif
+#  endif
+#  ifdef __cplusplus
+}
+#  endif
 # endif
 #endif /* ! defined (yyoverflow) || YYERROR_VERBOSE */
 
@@ -275,7 +329,7 @@ union yyalloc
 #   define YYCOPY(To, From, Count)		\
       do					\
 	{					\
-	  register YYSIZE_T yyi;		\
+	  YYSIZE_T yyi;				\
 	  for (yyi = 0; yyi < (Count); yyi++)	\
 	    (To)[yyi] = (From)[yyi];		\
 	}					\
@@ -325,7 +379,7 @@ union yyalloc
 #define YYUNDEFTOK  2
 #define YYMAXUTOK   285
 
-#define YYTRANSLATE(YYX) 						\
+#define YYTRANSLATE(YYX)						\
   ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
 
 /* YYTRANSLATE[YYLEX] -- Bison symbol number corresponding to YYLEX.  */
@@ -397,17 +451,17 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned short int yyrline[] =
 {
-       0,    78,    78,    78,    80,    82,    84,    88,    94,    98,
-     102,   107,   108,   108,   108,   108,   110,   114,   118,   122,
-     129,   136,   142,   151,   157,   168,   169,   169,   169,   169,
-     171,   176,   181,   186,   193,   198,   203,   208,   213,   217,
-     221,   228,   235,   242,   246,   252,   258,   264,   271,   280,
-     287,   296,   303,   311,   311
+       0,    74,    74,    74,    76,    78,    80,    84,    90,    94,
+      98,   103,   104,   104,   104,   104,   106,   110,   114,   118,
+     125,   132,   138,   147,   153,   164,   165,   165,   165,   165,
+     167,   172,   177,   182,   189,   194,   199,   204,   209,   213,
+     217,   224,   231,   238,   242,   248,   254,   260,   267,   276,
+     283,   292,   299,   307,   307
 };
 #endif
 
-#if YYDEBUG || YYERROR_VERBOSE
-/* YYTNME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
+#if YYDEBUG || YYERROR_VERBOSE || YYTOKEN_TABLE
+/* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
    First, the terminals, then, starting at YYNTOKENS, nonterminals. */
 static const char *const yytname[] =
 {
@@ -549,22 +603,6 @@ static const unsigned char yystos[] =
       33,    33
 };
 
-#if ! defined (YYSIZE_T) && defined (__SIZE_TYPE__)
-# define YYSIZE_T __SIZE_TYPE__
-#endif
-#if ! defined (YYSIZE_T) && defined (size_t)
-# define YYSIZE_T size_t
-#endif
-#if ! defined (YYSIZE_T)
-# if defined (__STDC__) || defined (__cplusplus)
-#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
-#  define YYSIZE_T size_t
-# endif
-#endif
-#if ! defined (YYSIZE_T)
-# define YYSIZE_T unsigned int
-#endif
-
 #define yyerrok		(yyerrstatus = 0)
 #define yyclearin	(yychar = YYEMPTY)
 #define YYEMPTY		(-2)
@@ -594,8 +632,8 @@ do								\
       goto yybackup;						\
     }								\
   else								\
-    { 								\
-      yyerror ("syntax error: cannot back up");\
+    {								\
+      yyerror (YY_("syntax error: cannot back up")); \
       YYERROR;							\
     }								\
 while (0)
@@ -674,7 +712,7 @@ do {								\
   if (yydebug)							\
     {								\
       YYFPRINTF (stderr, "%s ", Title);				\
-      yysymprint (stderr, 					\
+      yysymprint (stderr,					\
                   Type, Value);	\
       YYFPRINTF (stderr, "\n");					\
     }								\
@@ -722,13 +760,13 @@ yy_reduce_print (yyrule)
 #endif
 {
   int yyi;
-  unsigned int yylno = yyrline[yyrule];
-  YYFPRINTF (stderr, "Reducing stack by rule %d (line %u), ",
+  unsigned long int yylno = yyrline[yyrule];
+  YYFPRINTF (stderr, "Reducing stack by rule %d (line %lu), ",
              yyrule - 1, yylno);
   /* Print the symbols being reduced, and their result.  */
   for (yyi = yyprhs[yyrule]; 0 <= yyrhs[yyi]; yyi++)
-    YYFPRINTF (stderr, "%s ", yytname [yyrhs[yyi]]);
-  YYFPRINTF (stderr, "-> %s\n", yytname [yyr1[yyrule]]);
+    YYFPRINTF (stderr, "%s ", yytname[yyrhs[yyi]]);
+  YYFPRINTF (stderr, "-> %s\n", yytname[yyr1[yyrule]]);
 }
 
 # define YY_REDUCE_PRINT(Rule)		\
@@ -757,7 +795,7 @@ int yydebug;
    if the built-in stack extension method is used).
 
    Do not make this value too large; the results are undefined if
-   SIZE_MAX < YYSTACK_BYTES (YYMAXDEPTH)
+   YYSTACK_ALLOC_MAXIMUM < YYSTACK_BYTES (YYMAXDEPTH)
    evaluated with infinite-precision integer arithmetic.  */
 
 #ifndef YYMAXDEPTH
@@ -781,7 +819,7 @@ yystrlen (yystr)
      const char *yystr;
 #   endif
 {
-  register const char *yys = yystr;
+  const char *yys = yystr;
 
   while (*yys++ != '\0')
     continue;
@@ -806,8 +844,8 @@ yystpcpy (yydest, yysrc)
      const char *yysrc;
 #   endif
 {
-  register char *yyd = yydest;
-  register const char *yys = yysrc;
+  char *yyd = yydest;
+  const char *yys = yysrc;
 
   while ((*yyd++ = *yys++) != '\0')
     continue;
@@ -817,7 +855,55 @@ yystpcpy (yydest, yysrc)
 #  endif
 # endif
 
-#endif /* !YYERROR_VERBOSE */
+# ifndef yytnamerr
+/* Copy to YYRES the contents of YYSTR after stripping away unnecessary
+   quotes and backslashes, so that it's suitable for yyerror.  The
+   heuristic is that double-quoting is unnecessary unless the string
+   contains an apostrophe, a comma, or backslash (other than
+   backslash-backslash).  YYSTR is taken from yytname.  If YYRES is
+   null, do not copy; instead, return the length of what the result
+   would have been.  */
+static YYSIZE_T
+yytnamerr (char *yyres, const char *yystr)
+{
+  if (*yystr == '"')
+    {
+      size_t yyn = 0;
+      char const *yyp = yystr;
+
+      for (;;)
+	switch (*++yyp)
+	  {
+	  case '\'':
+	  case ',':
+	    goto do_not_strip_quotes;
+
+	  case '\\':
+	    if (*++yyp != '\\')
+	      goto do_not_strip_quotes;
+	    /* Fall through.  */
+	  default:
+	    if (yyres)
+	      yyres[yyn] = *yyp;
+	    yyn++;
+	    break;
+
+	  case '"':
+	    if (yyres)
+	      yyres[yyn] = '\0';
+	    return yyn;
+	  }
+    do_not_strip_quotes: ;
+    }
+
+  if (! yyres)
+    return yystrlen (yystr);
+
+  return yystpcpy (yyres, yystr) - yyres;
+}
+# endif
+
+#endif /* YYERROR_VERBOSE */
 
 
 
@@ -937,13 +1023,13 @@ yyparse (void)
 #else
 int
 yyparse ()
-
+    ;
 #endif
 #endif
 {
   
-  register int yystate;
-  register int yyn;
+  int yystate;
+  int yyn;
   int yyresult;
   /* Number of tokens to shift before error messages enabled.  */
   int yyerrstatus;
@@ -961,12 +1047,12 @@ yyparse ()
   /* The state stack.  */
   short int yyssa[YYINITDEPTH];
   short int *yyss = yyssa;
-  register short int *yyssp;
+  short int *yyssp;
 
   /* The semantic value stack.  */
   YYSTYPE yyvsa[YYINITDEPTH];
   YYSTYPE *yyvs = yyvsa;
-  register YYSTYPE *yyvsp;
+  YYSTYPE *yyvsp;
 
 
 
@@ -998,9 +1084,6 @@ yyparse ()
   yyssp = yyss;
   yyvsp = yyvs;
 
-
-  yyvsp[0] = yylval;
-
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -1033,7 +1116,7 @@ yyparse ()
 	   data in use in that stack, in bytes.  This used to be a
 	   conditional around just the two extra args, but that might
 	   be undefined if yyoverflow is a macro.  */
-	yyoverflow ("parser stack overflow",
+	yyoverflow (YY_("memory exhausted"),
 		    &yyss1, yysize * sizeof (*yyssp),
 		    &yyvs1, yysize * sizeof (*yyvsp),
 
@@ -1044,11 +1127,11 @@ yyparse ()
       }
 #else /* no yyoverflow */
 # ifndef YYSTACK_RELOCATE
-      goto yyoverflowlab;
+      goto yyexhaustedlab;
 # else
       /* Extend the stack our own way.  */
       if (YYMAXDEPTH <= yystacksize)
-	goto yyoverflowlab;
+	goto yyexhaustedlab;
       yystacksize *= 2;
       if (YYMAXDEPTH < yystacksize)
 	yystacksize = YYMAXDEPTH;
@@ -1058,7 +1141,7 @@ yyparse ()
 	union yyalloc *yyptr =
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
-	  goto yyoverflowlab;
+	  goto yyexhaustedlab;
 	YYSTACK_RELOCATE (yyss);
 	YYSTACK_RELOCATE (yyvs);
 
@@ -1187,63 +1270,63 @@ yyreduce:
   switch (yyn)
     {
         case 6:
-#line 84 "parser.y"
+#line 80 "parser.y"
     {
 			cfg_error("Invalid statement '%s'", (yyvsp[0].str));
 		}
     break;
 
   case 7:
-#line 88 "parser.y"
+#line 84 "parser.y"
     {
 			yyclearin; yyerrok;
 		}
     break;
 
   case 8:
-#line 94 "parser.y"
+#line 90 "parser.y"
     {
 				parser_device = &default_device;
 			}
     break;
 
   case 9:
-#line 98 "parser.y"
+#line 94 "parser.y"
     {
 				parser_device = alloc_device_opts((yyvsp[0].str));
 			}
     break;
 
   case 10:
-#line 102 "parser.y"
+#line 98 "parser.y"
     {
 				parser_device = alloc_device_opts((yyvsp[0].str));
 			}
     break;
 
   case 16:
-#line 110 "parser.y"
+#line 106 "parser.y"
     {
 				hcid.auto_init = (yyvsp[0].num);
 			}
     break;
 
   case 17:
-#line 114 "parser.y"
+#line 110 "parser.y"
     {
 				hcid.security = (yyvsp[0].num);
 			}
     break;
 
   case 18:
-#line 118 "parser.y"
+#line 114 "parser.y"
     {
 				hcid.pairing = (yyvsp[0].num);
 			}
     break;
 
   case 19:
-#line 122 "parser.y"
+#line 118 "parser.y"
     {
 				if (hcid.pin_helper)
 					free(hcid.pin_helper);
@@ -1253,7 +1336,7 @@ yyreduce:
     break;
 
   case 20:
-#line 129 "parser.y"
+#line 125 "parser.y"
     {
 				if (hcid.pin_helper)
 					free(hcid.pin_helper);
@@ -1263,14 +1346,14 @@ yyreduce:
     break;
 
   case 21:
-#line 136 "parser.y"
+#line 132 "parser.y"
     {
 				cfg_error("Unknown option '%s'", (yyvsp[0].str));
 			}
     break;
 
   case 22:
-#line 142 "parser.y"
+#line 138 "parser.y"
     {
 			int opt = find_keyword(sec_param, (yyvsp[0].str));
 			if (opt < 0) {
@@ -1282,14 +1365,14 @@ yyreduce:
     break;
 
   case 23:
-#line 151 "parser.y"
+#line 147 "parser.y"
     {
 			(yyval.num) = HCID_SEC_NONE;
 		}
     break;
 
   case 24:
-#line 157 "parser.y"
+#line 153 "parser.y"
     {
 			int opt = find_keyword(pair_param, (yyvsp[0].str));
 			if (opt < 0) {
@@ -1301,7 +1384,7 @@ yyreduce:
     break;
 
   case 30:
-#line 171 "parser.y"
+#line 167 "parser.y"
     {
 				parser_device->flags |= (1 << HCID_SET_PTYPE);
 				parser_device->pkt_type = (yyvsp[0].num);
@@ -1309,7 +1392,7 @@ yyreduce:
     break;
 
   case 31:
-#line 176 "parser.y"
+#line 172 "parser.y"
     {
 				parser_device->flags |= (1 << HCID_SET_LM);
 				parser_device->link_mode = (yyvsp[0].num);
@@ -1317,7 +1400,7 @@ yyreduce:
     break;
 
   case 32:
-#line 181 "parser.y"
+#line 177 "parser.y"
     {
 				parser_device->flags |= (1 << HCID_SET_LP);
 				parser_device->link_policy = (yyvsp[0].num);
@@ -1325,7 +1408,7 @@ yyreduce:
     break;
 
   case 33:
-#line 186 "parser.y"
+#line 182 "parser.y"
     {
 				if (parser_device->name)
 					free(parser_device->name);
@@ -1335,7 +1418,7 @@ yyreduce:
     break;
 
   case 34:
-#line 193 "parser.y"
+#line 189 "parser.y"
     {
 				parser_device->flags |= (1 << HCID_SET_CLASS);
 				parser_device->class = (yyvsp[0].num);
@@ -1343,7 +1426,7 @@ yyreduce:
     break;
 
   case 35:
-#line 198 "parser.y"
+#line 194 "parser.y"
     {
 				parser_device->flags |= (1 << HCID_SET_VOICE);
 				parser_device->voice = (yyvsp[0].num);
@@ -1351,7 +1434,7 @@ yyreduce:
     break;
 
   case 36:
-#line 203 "parser.y"
+#line 199 "parser.y"
     {
 				parser_device->flags |= (1 << HCID_SET_INQMODE);
 				parser_device->inqmode = (yyvsp[0].num);
@@ -1359,7 +1442,7 @@ yyreduce:
     break;
 
   case 37:
-#line 208 "parser.y"
+#line 204 "parser.y"
     {
 				parser_device->flags |= (1 << HCID_SET_PAGETO);
 				parser_device->pageto = (yyvsp[0].num);
@@ -1367,21 +1450,21 @@ yyreduce:
     break;
 
   case 38:
-#line 213 "parser.y"
+#line 209 "parser.y"
     {
 				parser_device->auth = (yyvsp[0].num);
 			}
     break;
 
   case 39:
-#line 217 "parser.y"
+#line 213 "parser.y"
     {
 				 parser_device->encrypt = (yyvsp[0].num);
 			}
     break;
 
   case 40:
-#line 221 "parser.y"
+#line 217 "parser.y"
     {
 				if ((yyvsp[0].num))
 					parser_device->scan |=  SCAN_INQUIRY;
@@ -1391,7 +1474,7 @@ yyreduce:
     break;
 
   case 41:
-#line 228 "parser.y"
+#line 224 "parser.y"
     {
 				if ((yyvsp[0].num))
 					parser_device->scan |=  SCAN_PAGE;
@@ -1401,7 +1484,7 @@ yyreduce:
     break;
 
   case 42:
-#line 235 "parser.y"
+#line 231 "parser.y"
     {
 				cfg_error("Unknown option '%s'",(yyvsp[0].str));
 				YYABORT;
@@ -1409,35 +1492,35 @@ yyreduce:
     break;
 
   case 43:
-#line 242 "parser.y"
+#line 238 "parser.y"
     {
 			(yyval.str) = strdup((yyvsp[0].str));
 		}
     break;
 
   case 44:
-#line 246 "parser.y"
+#line 242 "parser.y"
     {
 			(yyval.str) = strdup((yyvsp[0].str));
 		}
     break;
 
   case 45:
-#line 252 "parser.y"
+#line 248 "parser.y"
     {
 			(yyval.str) = strdup((yyvsp[0].str));
 		}
     break;
 
   case 46:
-#line 258 "parser.y"
+#line 254 "parser.y"
     {
 			(yyval.str) = strdup((yyvsp[0].str));
 		}
     break;
 
   case 47:
-#line 264 "parser.y"
+#line 260 "parser.y"
     {
 			unsigned int opt;
 			if (!hci_strtoptype((yyvsp[0].str), &opt))
@@ -1447,7 +1530,7 @@ yyreduce:
     break;
 
   case 48:
-#line 271 "parser.y"
+#line 267 "parser.y"
     {
 			unsigned int opt;
 			if (!hci_strtoptype((yyvsp[0].str), &opt))
@@ -1457,7 +1540,7 @@ yyreduce:
     break;
 
   case 49:
-#line 280 "parser.y"
+#line 276 "parser.y"
     {
 			unsigned int opt;
 			if (!hci_strtolm((yyvsp[0].str), &opt))
@@ -1467,7 +1550,7 @@ yyreduce:
     break;
 
   case 50:
-#line 287 "parser.y"
+#line 283 "parser.y"
     {
 			unsigned int opt;
 			if (!hci_strtolm((yyvsp[0].str), &opt))
@@ -1477,7 +1560,7 @@ yyreduce:
     break;
 
   case 51:
-#line 296 "parser.y"
+#line 292 "parser.y"
     {
 			unsigned int opt;
 			if (!hci_strtolp((yyvsp[0].str), &opt))
@@ -1487,7 +1570,7 @@ yyreduce:
     break;
 
   case 52:
-#line 303 "parser.y"
+#line 299 "parser.y"
     {
 			unsigned int opt;
 			if (!hci_strtolp((yyvsp[0].str), &opt))
@@ -1497,20 +1580,21 @@ yyreduce:
     break;
 
   case 53:
-#line 311 "parser.y"
+#line 307 "parser.y"
     { (yyval.num) = 1; }
     break;
 
   case 54:
-#line 311 "parser.y"
+#line 307 "parser.y"
     { (yyval.num) = 0; }
     break;
 
 
+      default: break;
     }
 
-/* Line 1037 of yacc.c.  */
-#line 1514 "parser.c"
+/* Line 1126 of yacc.c.  */
+#line 1598 "parser.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1549,12 +1633,36 @@ yyerrlab:
 
       if (YYPACT_NINF < yyn && yyn < YYLAST)
 	{
-	  YYSIZE_T yysize = 0;
 	  int yytype = YYTRANSLATE (yychar);
-	  const char* yyprefix;
-	  char *yymsg;
+	  YYSIZE_T yysize0 = yytnamerr (0, yytname[yytype]);
+	  YYSIZE_T yysize = yysize0;
+	  YYSIZE_T yysize1;
+	  int yysize_overflow = 0;
+	  char *yymsg = 0;
+#	  define YYERROR_VERBOSE_ARGS_MAXIMUM 5
+	  char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
 	  int yyx;
 
+#if 0
+	  /* This is so xgettext sees the translatable formats that are
+	     constructed on the fly.  */
+	  YY_("syntax error, unexpected %s");
+	  YY_("syntax error, unexpected %s, expecting %s");
+	  YY_("syntax error, unexpected %s, expecting %s or %s");
+	  YY_("syntax error, unexpected %s, expecting %s or %s or %s");
+	  YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s");
+#endif
+	  char *yyfmt;
+	  char const *yyf;
+	  static char const yyunexpected[] = "syntax error, unexpected %s";
+	  static char const yyexpecting[] = ", expecting %s";
+	  static char const yyor[] = " or %s";
+	  char yyformat[sizeof yyunexpected
+			+ sizeof yyexpecting - 1
+			+ ((YYERROR_VERBOSE_ARGS_MAXIMUM - 2)
+			   * (sizeof yyor - 1))];
+	  char const *yyprefix = yyexpecting;
+
 	  /* Start YYX at -YYN if negative to avoid negative indexes in
 	     YYCHECK.  */
 	  int yyxbegin = yyn < 0 ? -yyn : 0;
@@ -1562,48 +1670,68 @@ yyerrlab:
 	  /* Stay within bounds of both yycheck and yytname.  */
 	  int yychecklim = YYLAST - yyn;
 	  int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
-	  int yycount = 0;
+	  int yycount = 1;
+
+	  yyarg[0] = yytname[yytype];
+	  yyfmt = yystpcpy (yyformat, yyunexpected);
 
-	  yyprefix = ", expecting ";
 	  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 	    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
 	      {
-		yysize += yystrlen (yyprefix) + yystrlen (yytname [yyx]);
-		yycount += 1;
-		if (yycount == 5)
+		if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
 		  {
-		    yysize = 0;
+		    yycount = 1;
+		    yysize = yysize0;
+		    yyformat[sizeof yyunexpected - 1] = '\0';
 		    break;
 		  }
+		yyarg[yycount++] = yytname[yyx];
+		yysize1 = yysize + yytnamerr (0, yytname[yyx]);
+		yysize_overflow |= yysize1 < yysize;
+		yysize = yysize1;
+		yyfmt = yystpcpy (yyfmt, yyprefix);
+		yyprefix = yyor;
 	      }
-	  yysize += (sizeof ("syntax error, unexpected ")
-		     + yystrlen (yytname[yytype]));
-	  yymsg = (char *) YYSTACK_ALLOC (yysize);
-	  if (yymsg != 0)
-	    {
-	      char *yyp = yystpcpy (yymsg, "syntax error, unexpected ");
-	      yyp = yystpcpy (yyp, yytname[yytype]);
 
-	      if (yycount < 5)
+	  yyf = YY_(yyformat);
+	  yysize1 = yysize + yystrlen (yyf);
+	  yysize_overflow |= yysize1 < yysize;
+	  yysize = yysize1;
+
+	  if (!yysize_overflow && yysize <= YYSTACK_ALLOC_MAXIMUM)
+	    yymsg = (char *) YYSTACK_ALLOC (yysize);
+	  if (yymsg)
+	    {
+	      /* Avoid sprintf, as that infringes on the user's name space.
+		 Don't have undefined behavior even if the translation
+		 produced a string with the wrong number of "%s"s.  */
+	      char *yyp = yymsg;
+	      int yyi = 0;
+	      while ((*yyp = *yyf))
 		{
-		  yyprefix = ", expecting ";
-		  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
-		    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
-		      {
-			yyp = yystpcpy (yyp, yyprefix);
-			yyp = yystpcpy (yyp, yytname[yyx]);
-			yyprefix = " or ";
-		      }
+		  if (*yyp == '%' && yyf[1] == 's' && yyi < yycount)
+		    {
+		      yyp += yytnamerr (yyp, yyarg[yyi++]);
+		      yyf += 2;
+		    }
+		  else
+		    {
+		      yyp++;
+		      yyf++;
+		    }
 		}
 	      yyerror (yymsg);
 	      YYSTACK_FREE (yymsg);
 	    }
 	  else
-	    yyerror ("syntax error; also virtual memory exhausted");
+	    {
+	      yyerror (YY_("syntax error"));
+	      goto yyexhaustedlab;
+	    }
 	}
       else
 #endif /* YYERROR_VERBOSE */
-	yyerror ("syntax error");
+	yyerror (YY_("syntax error"));
     }
 
 
@@ -1615,18 +1743,9 @@ yyerrlab:
 
       if (yychar <= YYEOF)
         {
-          /* If at end of input, pop the error token,
-	     then the rest of the stack, then return failure.  */
+	  /* Return failure if at end of input.  */
 	  if (yychar == YYEOF)
-	     for (;;)
-	       {
-
-		 YYPOPSTACK;
-		 if (yyssp == yyss)
-		   YYABORT;
-		 yydestruct ("Error: popping",
-                             yystos[*yyssp], yyvsp);
-	       }
+	    YYABORT;
         }
       else
 	{
@@ -1645,12 +1764,11 @@ yyerrlab:
 `---------------------------------------------------*/
 yyerrorlab:
 
-#ifdef __GNUC__
-  /* Pacify GCC when the user code never invokes YYERROR and the label
-     yyerrorlab therefore never appears in user code.  */
+  /* Pacify compilers like GCC when the user code never invokes
+     YYERROR and the label yyerrorlab therefore never appears in user
+     code.  */
   if (0)
      goto yyerrorlab;
-#endif
 
 yyvsp -= yylen;
   yyssp -= yylen;
@@ -1713,23 +1831,29 @@ yyacceptlab:
 | yyabortlab -- YYABORT comes here.  |
 `-----------------------------------*/
 yyabortlab:
-  yydestruct ("Error: discarding lookahead",
-              yytoken, &yylval);
-  yychar = YYEMPTY;
   yyresult = 1;
   goto yyreturn;
 
 #ifndef yyoverflow
-/*----------------------------------------------.
-| yyoverflowlab -- parser overflow comes here.  |
-`----------------------------------------------*/
-yyoverflowlab:
-  yyerror ("parser stack overflow");
+/*-------------------------------------------------.
+| yyexhaustedlab -- memory exhaustion comes here.  |
+`-------------------------------------------------*/
+yyexhaustedlab:
+  yyerror (YY_("memory exhausted"));
   yyresult = 2;
   /* Fall through.  */
 #endif
 
 yyreturn:
+  if (yychar != YYEOF && yychar != YYEMPTY)
+     yydestruct ("Cleanup: discarding lookahead",
+		 yytoken, &yylval);
+  while (yyssp != yyss)
+    {
+      yydestruct ("Cleanup: popping",
+		  yystos[*yyssp], yyvsp);
+      YYPOPSTACK;
+    }
 #ifndef yyoverflow
   if (yyss != yyssa)
     YYSTACK_FREE (yyss);
@@ -1738,7 +1862,7 @@ yyreturn:
 }
 
 
-#line 313 "parser.y"
+#line 309 "parser.y"
 
 
 int yyerror(char *s) 
