@@ -4,70 +4,65 @@
  *
  *  Copyright (C) 2001-2002  Nokia Corporation
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *  Copyright (C) 2002-2003  Stephen Crane <steve.crane@rococosoft.com>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: main.c,v 1.6 2004/12/25 17:43:19 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
+#include <unistd.h>
+#include <stdlib.h>
 #include <signal.h>
 #include <syslog.h>
 #include <getopt.h>
-#include <sys/un.h>
-#include <sys/select.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
-#include <netinet/in.h>
 
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/l2cap.h>
 #include <bluetooth/sdp.h>
 #include <bluetooth/sdp_lib.h>
 
+#include <sys/un.h>
+#include <netinet/in.h>
+
 #include "sdpd.h"
 
 static int l2cap_sock, unix_sock;
 static fd_set active_fdset;
 static int active_maxfd;
 
-sdp_record_t *server;
+static sdp_record_t *server;
 
 /*
  * List of version numbers supported by the SDP server.
  * Add to this list when newer versions are supported.
  */
 static sdp_version_t sdpVnumArray[1] = {
-	{1, 0}
+	{ 1, 0 }
 };
-const int sdpServerVnumEntries = 1;
+static const int sdpServerVnumEntries = 1;
 
 /*
  * The service database state is an attribute of the service record
@@ -99,18 +94,21 @@ static void add_lang_attr(sdp_record_t *r)
 	sdp_list_free(langs, 0);
 }
 
-static void register_public_browse_group(void)
+static void register_public_browse_group(int public)
 {
 	sdp_list_t *browselist;
 	uuid_t bgscid, pbgid;
 	sdp_data_t *sdpdata;
 	sdp_record_t *browse = sdp_record_alloc();
 
-	browse->handle = sdp_next_handle();
-	if (browse->handle < 0x10000)
-		return;
+	if (public) {
+		browse->handle = sdp_next_handle();
+		if (browse->handle < 0x10000)
+			return;
+	} else
+		browse->handle = SDP_SERVER_RECORD_HANDLE + 1;
 
-	sdp_record_add(browse);
+	sdp_record_add(BDADDR_ANY, browse);
 	sdpdata = sdp_data_alloc(SDP_UINT32, &browse->handle);
 	sdp_attr_add(browse, SDP_ATTR_RECORD_HANDLE, sdpdata);
 
@@ -122,8 +120,10 @@ static void register_public_browse_group(void)
 	sdp_set_service_classes(browse, browselist);
 	sdp_list_free(browselist, 0);
 
-	sdp_uuid16_create(&pbgid, PUBLIC_BROWSE_GROUP);
-	sdp_set_group_id(browse, pbgid);
+	if (public) {
+		sdp_uuid16_create(&pbgid, PUBLIC_BROWSE_GROUP);
+		sdp_set_group_id(browse, pbgid);
+	}
 }
 
 /*
@@ -132,7 +132,7 @@ static void register_public_browse_group(void)
  * discovery clients. This method constructs a service record
  * and stores it in the repository
  */
-static void register_server_service(void)
+static void register_server_service(int public)
 {
 	int i;
 	sdp_list_t *classIDList, *browseList;
@@ -150,7 +150,7 @@ static void register_server_service(void)
 	/* Force the record to be SDP_SERVER_RECORD_HANDLE */
 	server->handle = SDP_SERVER_RECORD_HANDLE;
 
-	sdp_record_add(server);
+	sdp_record_add(BDADDR_ANY, server);
 	sdp_attr_add(server, SDP_ATTR_RECORD_HANDLE, sdp_data_alloc(SDP_UINT32, &server->handle));
 
 	/*
@@ -208,10 +208,12 @@ static void register_server_service(void)
 	sdp_data_free(version_data);
 	sdp_list_free(pd, 0);
 
-	sdp_uuid16_create(&browseGroupId, PUBLIC_BROWSE_GROUP);
-	browseList = sdp_list_append(0, &browseGroupId);
-	sdp_set_browse_groups(server, browseList);
-	sdp_list_free(browseList, 0);
+	if (public) {
+		sdp_uuid16_create(&browseGroupId, PUBLIC_BROWSE_GROUP);
+		browseList = sdp_list_append(0, &browseGroupId);
+		sdp_set_browse_groups(server, browseList);
+		sdp_list_free(browseList, 0);
+	}
 
 	update_db_timestamp();
 }
@@ -221,16 +223,16 @@ static void register_server_service(void)
  * l2cap and unix sockets over which discovery and registration clients
  * access us respectively
  */
-int init_server(int master)
+static int init_server(int master, int public)
 {
 	struct sockaddr_l2 l2addr;
 	struct sockaddr_un unaddr;
 
 	/* Register the public browse group root */
-	register_public_browse_group();
+	register_public_browse_group(public);
 
 	/* Register the SDP server's service record */
-	register_server_service();
+	register_server_service(public);
 
 	/* Create L2CAP socket */
 	l2cap_sock = socket(PF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_L2CAP);
@@ -278,7 +280,7 @@ int init_server(int master)
 	return 0;
 }
 
-void sig_term(int sig)
+static void sig_term(int sig)
 {
 	SDPINF("terminating... \n");
 	sdp_svcdb_reset();
@@ -287,7 +289,7 @@ void sig_term(int sig)
 	exit(0);
 }
 
-int become_daemon(void)
+static int become_daemon(void)
 {
 	int fd;
 
@@ -306,14 +308,14 @@ int become_daemon(void)
 	return 1;
 }
 
-static inline void handle_request(int sk, char *data, int len)
+static inline void handle_request(int sk, uint8_t *data, int len)
 {
 	struct sockaddr_l2 sa;
-	int size;
+	socklen_t size;
 	sdp_req_t req;
 
 	size = sizeof(sa);
-	if (getpeername(sk, (struct sockaddr *)&sa, &size) < 0)
+	if (getpeername(sk, (struct sockaddr *) &sa, &size) < 0)
 		return;
 
 	if (sa.l2_family == AF_BLUETOOTH) { 
@@ -321,13 +323,18 @@ static inline void handle_request(int sk, char *data, int len)
 		memset(&lo, 0, sizeof(lo));
 		size = sizeof(lo);
 		getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &lo, &size);
-		req.bdaddr = sa.l2_bdaddr;
-		req.mtu    = lo.omtu;
-		req.local  = 0;
+		bacpy(&req.bdaddr, &sa.l2_bdaddr);
+		req.mtu = lo.omtu;
+		req.local = 0;
+		memset(&sa, 0, sizeof(sa));
+		size = sizeof(sa);
+		getsockname(sk, (struct sockaddr *) &sa, &size);
+		bacpy(&req.device, &sa.l2_bdaddr);
 	} else {
-		req.bdaddr = *BDADDR_LOCAL;
-		req.mtu    = 2048;
-		req.local  = 1;
+		bacpy(&req.device, BDADDR_ANY);
+		bacpy(&req.bdaddr, BDADDR_LOCAL);
+		req.mtu = 2048;
+		req.local = 1;
 	}
 	req.sock = sk;
 	req.buf  = data;
@@ -350,7 +357,7 @@ static void check_active(fd_set *mask, int num)
 {
 	sdp_pdu_hdr_t hdr;
 	int size, fd, count, r;
-	char *buf;
+	uint8_t *buf;
 
 	for (fd = 0, count = 0; fd <= active_maxfd && count < num; fd++) {
 		if (fd == l2cap_sock || fd == unix_sock || !FD_ISSET(fd, mask))
@@ -377,7 +384,7 @@ static void check_active(fd_set *mask, int num)
 	}
 }
 
-void usage(void)
+static void usage(void)
 {
 	printf("sdpd version %s\n", VERSION);
 	printf("Usage:\n"
@@ -386,19 +393,21 @@ void usage(void)
 }
 
 static struct option main_options[] = {
-        {"help", 0,0, 'h'},
-        {"nodaemon",  0,0, 'n'},
-        {"master",  0,0, 'm'},
-        {0, 0, 0, 0}
+	{ "help",	0, 0, 'h' },
+	{ "nodaemon",	0, 0, 'n' },
+	{ "master",	0, 0, 'm' },
+	{ "public",	0, 0, 'p' },
+        { 0, 0, 0, 0}
 };
 
 int main(int argc, char **argv)
 {
 	int daemon = 1;
 	int master = 0;
+	int public = 0;
 	int opt;
 
-	while ((opt = getopt_long(argc, argv, "nm", main_options, NULL)) != -1)
+	while ((opt = getopt_long(argc, argv, "nmp", main_options, NULL)) != -1)
 		switch (opt) {
 		case 'n':
 			daemon = 0;
@@ -406,6 +415,9 @@ int main(int argc, char **argv)
 		case 'm':
 			master = 1;
 			break;
+		case 'p':
+			public = 1;
+			break;
 		default:
 			usage();
 			exit(0);
@@ -418,7 +430,7 @@ int main(int argc, char **argv)
 	argc -= optind;
 	argv += optind;
 
-	if (init_server(master) < 0) {
+	if (init_server(master, public) < 0) {
 		SDPERR("Server initialization failed");
 		return -1;
 	}
