@@ -4,44 +4,42 @@
  *
  *  Copyright (C) 2001-2002  Nokia Corporation
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *  Copyright (C) 2002-2003  Stephen Crane <steve.crane@rococosoft.com>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: request.c,v 1.4 2004/04/28 12:15:04 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <stdio.h>
+#include <errno.h>
 #include <malloc.h>
 #include <syslog.h>
-#include <netinet/in.h>
 #include <sys/socket.h>
 
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/sdp.h>
 #include <bluetooth/sdp_lib.h>
 
+#include <netinet/in.h>
+
 #include "sdpd.h"
 
 #define MIN(x, y) ((x) < (y))? (x): (y)
@@ -56,7 +54,7 @@
  * sequence. The data type of elements found in the
  * sequence is returned in the reference pDataType
  */
-static int extract_des(char *buf, sdp_list_t **svcReqSeq, uint8_t *pDataType, uint8_t expectedType)
+static int extract_des(uint8_t *buf, sdp_list_t **svcReqSeq, uint8_t *pDataType, uint8_t expectedType)
 {
 	uint8_t seqType;
 	int data_size = 0;
@@ -66,7 +64,7 @@ static int extract_des(char *buf, sdp_list_t **svcReqSeq, uint8_t *pDataType, ui
 	sdp_list_t *pSeq = NULL;
 	uint8_t dataType;
 	int status = 0;
-	const char *p;
+	const uint8_t *p;
 
 	SDPDBG("Seq type : %d\n", seqType);
 	if (!scanned || (seqType != SDP_SEQ8 && seqType != SDP_SEQ16)) {
@@ -141,7 +139,7 @@ static int extract_des(char *buf, sdp_list_t **svcReqSeq, uint8_t *pDataType, ui
 
 static int sdp_set_cstate_pdu(sdp_buf_t *buf, sdp_cont_state_t *cstate)
 {
-	char *pdata = buf->data + buf->data_size;
+	uint8_t *pdata = buf->data + buf->data_size;
 	int length = 0;
 
 	if (cstate) {
@@ -161,9 +159,9 @@ static int sdp_set_cstate_pdu(sdp_buf_t *buf, sdp_cont_state_t *cstate)
 	return length;
 }
 
-static sdp_cont_state_t *sdp_cstate_get(char *buffer)
+static sdp_cont_state_t *sdp_cstate_get(uint8_t *buffer)
 {
-	char *pdata = buffer;
+	uint8_t *pdata = buffer;
 	uint8_t cStateSize = *(uint8_t *)pdata;
 
 	/*
@@ -232,16 +230,16 @@ static int service_search_req(sdp_req_t *req, sdp_buf_t *buf)
 {
 	int status = 0, i, plen, mlen;
 	sdp_list_t *pattern = NULL;
-	int expected, actual;
+	uint16_t expected, actual;
 	uint8_t dtd;
 	sdp_cont_state_t *cstate = NULL;
-	char *pCacheBuffer = NULL;
+	uint8_t *pCacheBuffer = NULL;
 	int handleSize = 0;
-	long cStateId = -1;
+	uint32_t cStateId = 0;
 	short rsp_count = 0;
 	short *pTotalRecordCount, *pCurrentRecordCount;
 	int mtu;
-	char *pdata = req->buf + sizeof(sdp_pdu_hdr_t);
+	uint8_t *pdata = req->buf + sizeof(sdp_pdu_hdr_t);
 	int scanned = extract_des(pdata, &pattern, &dtd, SDP_TYPE_UUID);
 
 	SDPDBG("");
@@ -297,11 +295,12 @@ static int service_search_req(sdp_req_t *req, sdp_buf_t *buf)
 
 		handleSize = 0;
 		for (; list && rsp_count < expected; list = list->next) {
-			sdp_record_t *rec = (sdp_record_t *)list->data;
+			sdp_record_t *rec = (sdp_record_t *) list->data;
 
 			SDPDBG("Checking svcRec : 0x%x\n", rec->handle);
 				
-			if (sdp_match_uuid(pattern, rec->pattern) > 0) {
+			if (sdp_match_uuid(pattern, rec->pattern) > 0 &&
+					sdp_check_access(rec->handle, &req->device)) {
 				rsp_count++;
 				sdp_put_unaligned(htonl(rec->handle), (uint32_t *)pdata);
 				pdata += sizeof(uint32_t);
@@ -330,7 +329,7 @@ static int service_search_req(sdp_req_t *req, sdp_buf_t *buf)
 	}
 
 	/* under both the conditions below, the rsp buffer is not built yet */
-	if (cstate || cStateId != -1) {
+	if (cstate || cStateId > 0) {
 		short lastIndex = 0;
 
 		if (cstate) {
@@ -360,7 +359,7 @@ static int service_search_req(sdp_req_t *req, sdp_buf_t *buf)
 		 * current record count and increment the cached
 		 * buffer pointer to beyond the counters
 		 */
-		pdata = (char *)pCurrentRecordCount + sizeof(uint16_t);
+		pdata = (uint8_t *) pCurrentRecordCount + sizeof(uint16_t);
 
 		/* increment beyond the totalCount and the currentCount */
 		pCacheBuffer += 2 * sizeof(uint16_t);
@@ -485,14 +484,14 @@ static int extract_attrs(sdp_record_t *rec, sdp_list_t *seq, uint8_t dtd, sdp_bu
 static int service_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 {
 	sdp_cont_state_t *cstate = NULL;
-	char *pResponse = NULL;
+	uint8_t *pResponse = NULL;
 	short cstate_size = 0;
 	sdp_list_t *seq = NULL;
 	uint8_t dtd = 0;
 	int scanned = 0;
 	int max_rsp_size;
 	int status = 0, plen, mlen;
-	char *pdata = req->buf + sizeof(sdp_pdu_hdr_t);
+	uint8_t *pdata = req->buf + sizeof(sdp_pdu_hdr_t);
 	uint32_t handle = ntohl(sdp_get_unaligned((uint32_t *)pdata));
 
 	SDPDBG("");
@@ -604,7 +603,7 @@ done:
 static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 {
 	int status = 0, plen, totscanned;
-	char *pdata, *pResponse = NULL;
+	uint8_t *pdata, *pResponse = NULL;
 	int scanned, max, rsp_count = 0;
 	sdp_list_t *pattern = NULL, *seq = NULL, *svcList;
 	sdp_cont_state_t *cstate = NULL;
@@ -652,7 +651,7 @@ static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 
 	svcList = sdp_get_record_list();
 
-	tmpbuf.data = (char *)malloc(USHRT_MAX);
+	tmpbuf.data = malloc(USHRT_MAX);
 	tmpbuf.data_size = 0;
 	tmpbuf.buf_size = USHRT_MAX;
 	memset(tmpbuf.data, 0, USHRT_MAX);
@@ -671,8 +670,9 @@ static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 		/* no continuation state -> create new response */
 		sdp_list_t *p;
 		for (p = svcList; p; p = p->next) {
-			sdp_record_t *rec = (sdp_record_t *)p->data;
-			if (sdp_match_uuid(pattern, rec->pattern) > 0) {
+			sdp_record_t *rec = (sdp_record_t *) p->data;
+			if (sdp_match_uuid(pattern, rec->pattern) > 0 &&
+					sdp_check_access(rec->handle, &req->device)) {
 				rsp_count++;
 				status = extract_attrs(rec, seq, dtd, &tmpbuf);
 
@@ -764,13 +764,13 @@ void process_request(sdp_req_t *req)
 	sdp_pdu_hdr_t *reqhdr = (sdp_pdu_hdr_t *)req->buf;
 	sdp_pdu_hdr_t *rsphdr;
 	sdp_buf_t rsp;
-	char *buf = (char *)malloc(USHRT_MAX);
+	uint8_t *buf = malloc(USHRT_MAX);
 	int sent = 0;
 	int status = SDP_INVALID_SYNTAX;
 
 	SDPDBG("");
 
-	memset((void *)buf, 0, USHRT_MAX);
+	memset(buf, 0, USHRT_MAX);
 	rsp.data = buf + sizeof(sdp_pdu_hdr_t);
 	rsp.data_size = 0;
 	rsp.buf_size = USHRT_MAX - sizeof(sdp_pdu_hdr_t);
