@@ -4,48 +4,46 @@
  *
  *  Copyright (C) 2001-2002  Nokia Corporation
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *  Copyright (C) 2002-2003  Stephen Crane <steve.crane@rococosoft.com>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: service.c,v 1.5 2004/10/01 11:40:15 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <netinet/in.h>
+#include <stdio.h>
+#include <errno.h>
 #include <sys/socket.h>
 
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/sdp.h>
 #include <bluetooth/sdp_lib.h>
 
+#include <netinet/in.h>
+
 #include "sdpd.h"
 
 extern void update_db_timestamp(void);
 
 // FIXME: refactor for server-side
-static sdp_record_t *extract_pdu_server(char *p, uint32_t handleExpected, int *scanned)
+static sdp_record_t *extract_pdu_server(bdaddr_t *device, uint8_t *p, uint32_t handleExpected, int *scanned)
 {
 	int extractStatus = -1, localExtractedLength = 0;
 	uint8_t dtd;
@@ -70,15 +68,15 @@ static sdp_record_t *extract_pdu_server(char *p, uint32_t handleExpected, int *s
 	} else if (handleExpected != 0xffffffff)
 		rec = sdp_record_find(handleExpected);
 
-	if (rec == NULL) {
+	if (!rec) {
 		rec = sdp_record_alloc();
 		rec->attrlist = NULL;
 		if (lookAheadAttrId == SDP_ATTR_RECORD_HANDLE) {
 			rec->handle = handle;
-			sdp_record_add(rec);
+			sdp_record_add(device, rec);
 		} else if (handleExpected != 0xffffffff) {
 			rec->handle = handleExpected;
-			sdp_record_add(rec);
+			sdp_record_add(device, rec);
 		}
 	}
 
@@ -128,14 +126,18 @@ int service_register_req(sdp_req_t *req, sdp_buf_t *rsp)
 {
 	int scanned = 0;
 	sdp_data_t *handle;
-	char *p = req->buf + sizeof(sdp_pdu_hdr_t);
+	uint8_t *p = req->buf + sizeof(sdp_pdu_hdr_t);
 	sdp_record_t *rec;
 
 	req->flags = *p++;
+	if (req->flags & SDP_DEVICE_RECORD) {
+		bacpy(&req->device, (bdaddr_t *) p);
+		p += sizeof(bdaddr_t);
+	}
 
 	// save image of PDU: we need it when clients request this attribute
-	rec = extract_pdu_server(p, 0xffffffff, &scanned);
-	if (rec == NULL) {
+	rec = extract_pdu_server(&req->device, p, 0xffffffff, &scanned);
+	if (!rec) {
 		sdp_put_unaligned(htons(SDP_INVALID_SYNTAX), (uint16_t *)rsp->data);
 		rsp->data_size = sizeof(uint16_t);
 		return -1;
@@ -145,7 +147,7 @@ int service_register_req(sdp_req_t *req, sdp_buf_t *rsp)
 	if (rec->handle < 0x10000)
 		return -1;
 
-	sdp_record_add(rec);
+	sdp_record_add(&req->device, rec);
 	if (!(req->flags & SDP_RECORD_PERSIST))
 		sdp_svcdb_set_collectable(rec, req->sock);
 	handle = sdp_data_alloc(SDP_UINT32, &rec->handle);
@@ -174,7 +176,7 @@ int service_update_req(sdp_req_t *req, sdp_buf_t *rsp)
 {
 	sdp_record_t *orec;
 	int status = 0, scanned = 0;
-	char *p = req->buf + sizeof(sdp_pdu_hdr_t);
+	uint8_t *p = req->buf + sizeof(sdp_pdu_hdr_t);
 	uint32_t handle = ntohl(sdp_get_unaligned((uint32_t *)p));
 
 	SDPDBG("");
@@ -188,7 +190,7 @@ int service_update_req(sdp_req_t *req, sdp_buf_t *rsp)
 	SDPDBG("SvcRecOld: 0x%x\n", (uint32_t)orec);
 
 	if (orec) {
-		sdp_record_t *nrec = extract_pdu_server(p, handle, &scanned);
+		sdp_record_t *nrec = extract_pdu_server(BDADDR_ANY, p, handle, &scanned);
 		if (nrec && handle == nrec->handle)
 			update_db_timestamp();
 		else {
@@ -216,7 +218,7 @@ int service_update_req(sdp_req_t *req, sdp_buf_t *rsp)
  */
 int service_remove_req(sdp_req_t *req, sdp_buf_t *rsp)
 {
-	char *p = req->buf + sizeof(sdp_pdu_hdr_t);
+	uint8_t *p = req->buf + sizeof(sdp_pdu_hdr_t);
 	uint32_t handle = ntohl(sdp_get_unaligned((uint32_t *)p));
 	sdp_record_t *rec;
 	int status = 0;
