@@ -8,24 +8,19 @@
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: l2test.c,v 1.24 2005/01/28 17:24:40 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,6 +36,7 @@
 #include <syslog.h>
 #include <signal.h>
 #include <sys/time.h>
+#include <sys/poll.h>
 #include <sys/socket.h>
 
 #include <bluetooth/bluetooth.h>
@@ -71,7 +67,7 @@ static int imtu = 672;
 static int omtu = 0;
 
 /* Default data size */
-static long data_size = 0;
+static long data_size = -1;
 
 /* Default addr and psm */
 static bdaddr_t bdaddr;
@@ -80,6 +76,7 @@ static unsigned short psm = 10;
 /* Default number of frames to send (-1 = infinite) */
 static int num_frames = -1;
 
+static int flowctl = 0;
 static int master = 0;
 static int auth = 0;
 static int encrypt = 0;
@@ -129,7 +126,7 @@ static char *ctoh(char c, char* s)
 	return s;
 }
 
-static void hexdump(char *s, unsigned long l)
+static void hexdump(unsigned char *s, unsigned long l)
 {
 	char bfr[80];
 	char *pb;
@@ -205,6 +202,8 @@ static int do_connect(char *svr)
 	/* Set new options */
 	opts.omtu = omtu;
 	opts.imtu = imtu;
+	if (flowctl)
+		opts.mode = 2;
 
 	if (setsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &opts, sizeof(opts)) < 0) {
 		syslog(LOG_ERR, "Can't set L2CAP options: %s (%d)",
@@ -271,6 +270,9 @@ static int do_connect(char *svr)
 		opts.imtu, opts.omtu, opts.flush_to, opts.mode, conn.hci_handle,
 		conn.dev_class[2], conn.dev_class[1], conn.dev_class[0]);
 
+	if (data_size > opts.omtu)
+		data_size = opts.omtu;
+
 	return sk;
 
 error:
@@ -337,6 +339,8 @@ static void do_listen(void (*handler)(int sk))
 
 	/* Set new options */
 	opts.imtu = imtu;
+	if (flowctl)
+		opts.mode = 2;
 
 	if (setsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &opts, sizeof(opts)) < 0) {
 		syslog(LOG_ERR, "Can't set L2CAP options: %s (%d)",
@@ -468,13 +472,14 @@ static void dump_mode(int sk)
 		}
 
 		syslog(LOG_INFO, "Recevied %d bytes", len);
-		hexdump(buf,len);
+		hexdump(buf, len);
 	}
 }
 
 static void recv_mode(int sk)
 {
-	struct timeval tv_beg,tv_end,tv_diff;
+	struct timeval tv_beg, tv_end, tv_diff;
+	struct pollfd p;
 	long total;
 	uint32_t seq;
 	socklen_t optlen;
@@ -482,6 +487,9 @@ static void recv_mode(int sk)
 
 	syslog(LOG_INFO,"Receiving ...");
 
+	p.fd = sk;
+	p.events = POLLIN | POLLERR | POLLHUP;
+
 	seq = 0;
 	while (1) {
 		gettimeofday(&tv_beg, NULL);
@@ -489,27 +497,35 @@ static void recv_mode(int sk)
 		while (total < data_size) {
 			uint32_t sq;
 			uint16_t l;
-			int i, r;
+			int i, len;
 
-			if ((r = recv(sk, buf, data_size, 0)) <= 0) {
-				if (r < 0) {
-					if (reliable && (errno == ECOMM)) {
-						syslog(LOG_INFO, "L2CAP Error ECOMM - clearing error and continuing.\n");
-						optlen = sizeof(opt);
-						if (getsockopt(sk, SOL_SOCKET, SO_ERROR, &opt, &optlen) < 0) {
-							syslog(LOG_ERR, "Couldn't getsockopt(SO_ERROR): %s (%d)",
-								strerror(errno), errno);
-							return;
-						}
-						continue;
-					} else {
-						syslog(LOG_ERR, "Read failed: %s (%d)",
-							strerror(errno), errno);
-					}
-				}
+			p.revents = 0;
+			if (poll(&p, 1, -1) <= 0)
 				return;
+
+			if (p.revents & (POLLERR | POLLHUP))
+				return;
+
+			len = recv(sk, buf, data_size, 0);
+			if (len < 0) {
+				if (reliable && (errno == ECOMM)) {
+					syslog(LOG_INFO, "L2CAP Error ECOMM - clearing error and continuing.\n");
+					optlen = sizeof(opt);
+					if (getsockopt(sk, SOL_SOCKET, SO_ERROR, &opt, &optlen) < 0) {
+						syslog(LOG_ERR, "Couldn't getsockopt(SO_ERROR): %s (%d)",
+							strerror(errno), errno);
+						return;
+					}
+					continue;
+				} else {
+					syslog(LOG_ERR, "Read failed: %s (%d)",
+						strerror(errno), errno);
+				}
 			}
 
+			if (len < 6)
+				break;
+
 			/* Check sequence */
 			sq = btohl(*(uint32_t *) buf);
 			if (seq != sq) {
@@ -520,18 +536,18 @@ static void recv_mode(int sk)
 
 			/* Check length */
 			l = btohs(*(uint16_t *) (buf + 4));
-			if (r != l) {
-				syslog(LOG_INFO, "size missmatch: %d -> %d", r, l);
+			if (len != l) {
+				syslog(LOG_INFO, "size missmatch: %d -> %d", len, l);
 				continue;
 			}
 
 			/* Verify data */
-			for (i = 6; i < r; i++) {
+			for (i = 6; i < len; i++) {
 				if (buf[i] != 0x7f)
 					syslog(LOG_INFO, "data missmatch: byte %d 0x%2.2x", i, buf[i]);
 			}
 
-			total += r;
+			total += len;
 		}
 		gettimeofday(&tv_end, NULL);
 
@@ -545,7 +561,7 @@ static void recv_mode(int sk)
 static void send_mode(int sk)
 {
 	uint32_t seq;
-	int i;
+	int i, len;
 
 	syslog(LOG_INFO, "Sending ...");
 
@@ -558,7 +574,8 @@ static void send_mode(int sk)
 		*(uint16_t *) (buf + 4) = htobs(data_size);
 		seq++;
 
-		if (send(sk, buf, data_size, 0) <= 0) {
+		len = send(sk, buf, data_size, 0);
+		if (len < 0 || len != data_size) {
 			syslog(LOG_ERR, "Send failed: %s (%d)", strerror(errno), errno);
 			exit(1);
 		}
@@ -606,12 +623,24 @@ static void reconnect_mode(char *svr)
 
 static void connect_mode(char *svr)
 {
+	struct pollfd p;
 	int sk;
 
 	if ((sk = do_connect(svr)) < 0)
 		exit(1);
 
-	sleep(99999999);
+	p.fd = sk;
+	p.events = POLLERR | POLLHUP;
+
+	while (1) {
+		p.revents = 0;
+		if (poll(&p, 1, 100))
+			break;
+	}
+
+	syslog(LOG_INFO, "Disconnected");
+
+	close(sk);
 }
 
 static void multi_connect_mode(char *svr)
@@ -655,6 +684,7 @@ static void usage(void)
 		"\t[-L seconds] enable SO_LINGER\n"
 		"\t[-R] reliable mode\n"
 		"\t[-D] use connectionless channel (datagram)\n"
+		"\t[-F] enable flow control\n"
 		"\t[-A] request authentication\n"
 		"\t[-E] request encryption\n"
 		"\t[-S] secure connection\n"
@@ -668,7 +698,7 @@ int main(int argc ,char *argv[])
 
 	bacpy(&bdaddr, BDADDR_ANY);
 
-	while ((opt=getopt(argc,argv,"rdscuwmnxyb:i:P:I:O:N:RMAESL:D")) != EOF) {
+	while ((opt=getopt(argc,argv,"rdscuwmnxyb:i:P:I:O:N:L:RDFAESM")) != EOF) {
 		switch(opt) {
 		case 'r':
 			mode = RECV;
@@ -750,6 +780,10 @@ int main(int argc ,char *argv[])
 			master = 1;
 			break;
 
+		case 'F':
+			flowctl = 1;
+			break;
+
 		case 'A':
 			auth = 1;
 			break;
@@ -781,7 +815,8 @@ int main(int argc ,char *argv[])
 		exit(1);
 	}
 
-	if (!data_size) {
+	if (data_size < 0) {
+		data_size = 48;
 		if (imtu > data_size)
 			data_size = imtu;
 		if (omtu > data_size)
