@@ -2,28 +2,23 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: ciptool.c,v 1.6 2004/12/25 17:43:20 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -134,7 +129,8 @@ static int do_connect(int ctl, int dev_id, bdaddr_t *src, bdaddr_t *dst, unsigne
 	struct hci_dev_info di;
 	struct sockaddr_l2 addr;
 	struct l2cap_options opts;
-	int sk, size;
+	socklen_t size;
+	int sk;
 
 	hci_devinfo(dev_id, &di);
 	if (!(di.link_policy & HCI_LP_RSWITCH)) {
@@ -255,7 +251,7 @@ static void cmd_search(int ctl, bdaddr_t *bdaddr, int argc, char **argv)
 			if (!get_psm(&src, &dst, &psm))
 				continue;
 
-			free(info);
+			bt_free(info);
 
 			printf("\tConnecting to device %s\n", addr);
 			do_connect(ctl, dev_id, &src, &dst, psm, 0);
@@ -263,7 +259,7 @@ static void cmd_search(int ctl, bdaddr_t *bdaddr, int argc, char **argv)
 		}
 	}
 
-	free(info);
+	bt_free(info);
 	fprintf(stderr, "\tNo devices in range or visible\n");
 	exit(1);
 }
