@@ -8,24 +8,19 @@
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: hciattach.c,v 1.30 2005/06/21 16:49:26 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -649,11 +644,11 @@ static int csr(int fd, struct uart_t *u, struct termios *ti)
 
 /* 
  * Silicon Wave specific initialization 
- * Thomas Moser <Thomas.Moser@tmoser.ch>
+ * Thomas Moser <thomas.moser@tmoser.ch>
  */
 static int swave(int fd, struct uart_t *u, struct termios *ti)
 {
-	struct timespec tm = {0, 500000};
+	struct timespec tm = { 0, 500000 };
 	char cmd[10], rsp[100];
 	int r;
 
@@ -693,7 +688,7 @@ static int swave(int fd, struct uart_t *u, struct termios *ti)
 	}
 
 	/* Send initialization command */
-	if (write(fd, cmd, 9) != 9) {
+	if (write(fd, cmd, 10) != 10) {
 		perror("Failed to write init command");
 		return -1;
 	}
@@ -725,15 +720,16 @@ static int swave(int fd, struct uart_t *u, struct termios *ti)
 		return -1;
 	}
 
-	// we probably got the reply. Now we must send the "soft reset":
+	// we probably got the reply. Now we must send the "soft reset"
+	// which is standard HCI RESET.
+
 	cmd[0] = HCI_COMMAND_PKT;	// it's a command packet
-	cmd[1] = 0x0B;			// OCF 0x0B	= param access set	
-	cmd[2] = 0xfc;			// OGF bx111111 = vendor specific
-	cmd[3] = 0x01;			// 1 byte of data following 
-	cmd[4] = 0x03;			// HCI Reset Subcommand
-			
-	// Send initialization command
-	if (write(fd, cmd, 5) != 5) {
+	cmd[1] = 0x03;
+	cmd[2] = 0x0c;
+	cmd[3] = 0x00;
+
+	/* Send reset command */
+	if (write(fd, cmd, 4) != 4) {
 		perror("Can't write Silicon Wave reset cmd.");
 		return -1;
 	}
@@ -973,7 +969,7 @@ int main(int argc, char *argv[])
 	int send_break = 0;
 	pid_t pid;
 	struct sigaction sa;
-	char dev[20];
+	char dev[PATH_MAX];
 
 	detach = 1;
 	printpid = 0;
