@@ -8,24 +8,19 @@
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: hciconfig.c,v 1.69 2005/09/10 00:34:21 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,6 +32,7 @@
 #include <ctype.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <string.h>
 #include <getopt.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
@@ -158,7 +154,11 @@ static void cmd_iac(int ctl, int hdev, char *opt)
 	if (opt) {
 		int l = strtoul(opt, 0, 16);
 		uint8_t lap[3];
-		if (l < 0x9e8b00 || l > 0x9e8b3f) {
+		if (!strcasecmp(opt, "giac")) {
+			l = 0x9e8b33;
+		} else if (!strcasecmp(opt, "liac")) {
+			l = 0x9e8b00;
+		} else if (l < 0x9e8b00 || l > 0x9e8b3f) {
 			printf("Invalid access code 0x%x\n", l);
 			exit(1);
 		}
@@ -182,8 +182,8 @@ static void cmd_iac(int ctl, int hdev, char *opt)
 		for (i = 0; i < n; i++) {
 			printf("0x");
 			for (j = 3; j--; )
-				printf("%02x", lap[j + 3*i]);
-			if (i < n-1)
+				printf("%02x", lap[j + 3 * i]);
+			if (i < n - 1)
 				printf(", ");
 		}
 		printf("\n");
@@ -348,8 +348,7 @@ static void cmd_aclmtu(int ctl, int hdev, char *opt)
 	if (sscanf(opt, "%4hu:%4hu", &mtu, &mpkt) != 2)
 		return;
 
-	*((uint16_t *)&dr.dev_opt + 1) = mtu;
-	*((uint16_t *)&dr.dev_opt + 0) = mpkt;
+	dr.dev_opt = htobl(htobs(mpkt) | (htobs(mtu) << 16));
 
 	if (ioctl(ctl, HCISETACLMTU, (unsigned long) &dr) < 0) {
 		fprintf(stderr, "Can't set ACL mtu on hci%d: %s(%d)\n",
@@ -369,9 +368,8 @@ static void cmd_scomtu(int ctl, int hdev, char *opt)
 	if (sscanf(opt, "%4hu:%4hu", &mtu, &mpkt) != 2)
 		return;
 
-	*((uint16_t *)&dr.dev_opt + 1) = mtu;
-	*((uint16_t *)&dr.dev_opt + 0) = mpkt;
-	
+	dr.dev_opt = htobl(htobs(mpkt) | (htobs(mtu) << 16));
+
 	if (ioctl(ctl, HCISETSCOMTU, (unsigned long) &dr) < 0) {
 		fprintf(stderr, "Can't set SCO mtu on hci%d: %s (%d)\n",
 						hdev, strerror(errno), errno);
@@ -429,34 +427,66 @@ static void cmd_features(int ctl, int hdev, char *opt)
 
 static void cmd_name(int ctl, int hdev, char *opt)
 {
-	int s = hci_open_dev(hdev);
+	int dd;
 
-	if (s < 0) {
+	dd = hci_open_dev(hdev);
+	if (dd < 0) {
 		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
 						hdev, strerror(errno), errno);
 		exit(1);
 	}
+
 	if (opt) {
-		if (hci_write_local_name(s, opt, 2000) < 0) {
+		uint8_t fec = 0, data[240];
+		int len, eir = 0;
+
+		if (di.features[6] & LMP_EXT_INQ) {
+			if (hci_read_ext_inquiry_response(dd, &fec, data, 1000) == 0)
+				eir = 1;
+		}
+
+		memset(data, 0, sizeof(data));
+		len = strlen(opt);
+		if (len > 48) {
+			len = 48;
+			data[1] = 0x08;
+		} else
+			data[1] = 0x09;
+		data[0] = len + 1;
+		memcpy(data + 2, opt, len);
+
+		if (hci_write_local_name(dd, opt, 2000) < 0) {
 			fprintf(stderr, "Can't change local name on hci%d: %s (%d)\n",
 						hdev, strerror(errno), errno);
 			exit(1);
 		}
+
+		if (eir) {
+			if (hci_write_ext_inquiry_response(dd, fec, data, 2000) < 0) {
+				fprintf(stderr, "Can't set extended inquiry response on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+			}
+		}
 	} else {
 		char name[249];
 		int i;
-		if (hci_read_local_name(s, sizeof(name), name, 1000) < 0) {
+
+		if (hci_read_local_name(dd, sizeof(name), name, 1000) < 0) {
 			fprintf(stderr, "Can't read local name on hci%d: %s (%d)\n",
 						hdev, strerror(errno), errno);
 			exit(1);
 		}
+
 		for (i = 0; i < 248 && name[i]; i++)
 			if (!isprint(name[i]))
 				name[i] = '.';
 		name[248] = '\0';
+
 		print_dev_hdr(&di);
 		printf("\tName: '%s'\n", name);
 	}
+
+	hci_close_dev(dd);
 }
 
 /* 
@@ -913,6 +943,90 @@ static void cmd_inq_mode(int ctl, int hdev, char *opt)
 			break;
 		}
 	}
+
+	hci_close_dev(dd);
+}
+
+static void cmd_inq_data(int ctl, int hdev, char *opt)
+{
+	int i, dd;
+
+	dd = hci_open_dev(hdev);
+	if (dd < 0) {
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	if (opt) {
+		uint8_t fec = 0, data[240];
+		char tmp[3];
+		int i, size;
+
+		memset(data, 0, sizeof(data));
+
+		memset(tmp, 0, sizeof(tmp));
+		size = (strlen(opt) + 1) / 2;
+		if (size > 240)
+			size = 240;
+
+		for (i = 0; i < size; i++) {
+			memcpy(tmp, opt + (i * 2), 2);
+			data[i] = strtol(tmp, NULL, 16);
+		}
+
+		if (hci_write_ext_inquiry_response(dd, fec, data, 2000) < 0) {
+			fprintf(stderr, "Can't set extended inquiry response on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+			exit(1);
+		}
+	} else {
+		uint8_t fec, data[240], len, type, *ptr;
+		char *str;
+
+		if (hci_read_ext_inquiry_response(dd, &fec, data, 1000) < 0) {
+			fprintf(stderr, "Can't read extended inquiry response on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+			exit(1);
+		}
+
+		print_dev_hdr(&di);
+		printf("\tFEC %s\n\t\t", fec ? "enabled" : "disabled");
+		for (i = 0; i < 240; i++)
+			printf("%02x%s%s", data[i], (i + 1) % 8 ? "" : " ",
+				(i + 1) % 16 ? " " : (i < 239 ? "\n\t\t" : "\n"));
+
+		ptr = data;
+		while (*ptr) {
+			len = *ptr++;
+			type = *ptr++;
+			switch (type) {
+			case 0x08:
+			case 0x09:
+				str = malloc(len);
+				if (str) {
+					snprintf(str, len, "%s", ptr);
+					for (i = 0; i < len - 1; i++)
+						if (!isprint(str[i]))
+							str[i] = '.';
+					printf("\t%s local name: \'%s\'\n",
+						type == 0x08 ? "Shortened" : "Complete", str);
+					free(str);
+				}
+				break;
+			default:
+				printf("\tUnknown type 0x%02x with %d bytes data\n",
+								type, len - 1);
+				break;
+			}
+
+			ptr += (len - 1);
+		}
+
+		printf("\n");
+	}
+
+	hci_close_dev(dd);
 }
 
 static void cmd_inq_type(int ctl, int hdev, char *opt)
@@ -947,6 +1061,8 @@ static void cmd_inq_type(int ctl, int hdev, char *opt)
 		printf("\tInquiry scan type: %s\n",
 			type == 1 ? "Interlaced Inquiry Scan" : "Standard Inquiry Scan");
 	}
+
+	hci_close_dev(dd);
 }
 
 static void cmd_inq_parms(int ctl, int hdev, char *opt)
@@ -1382,8 +1498,9 @@ static struct {
 	{ "class",	cmd_class,	"[class]",	"Get/Set class of device" },
 	{ "voice",	cmd_voice,	"[voice]",	"Get/Set voice setting" },
 	{ "iac",	cmd_iac,	"[iac]",	"Get/Set inquiry access code" },
-	{ "inqmode",	cmd_inq_mode,	"[mode]",	"Get/set inquiry mode" },
-	{ "inqtype",	cmd_inq_type,	"[type]",	"Get/set inquiry scan type" },
+	{ "inqmode",	cmd_inq_mode,	"[mode]",	"Get/Set inquiry mode" },
+	{ "inqdata",	cmd_inq_data,	"[data]",	"Get/Set inquiry data" },
+	{ "inqtype",	cmd_inq_type,	"[type]",	"Get/Set inquiry scan type" },
 	{ "inqparms",	cmd_inq_parms,	"[win:int]",	"Get/Set inquiry scan window and interval" },
 	{ "pageparms",	cmd_page_parms,	"[win:int]",	"Get/Set page scan window and interval" },
 	{ "pageto",	cmd_page_to,	"[to]",		"Get/Set page timeout" },
