@@ -8,24 +8,19 @@
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: hcitool.c,v 1.80 2005/09/07 16:15:39 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,6 +33,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <string.h>
 #include <getopt.h>
 #include <sys/param.h>
 #include <sys/ioctl.h>
@@ -323,6 +319,7 @@ static struct option inq_options[] = {
 	{ "help",	0, 0, 'h' },
 	{ "length",	1, 0, 'l' },
 	{ "numrsp",	1, 0, 'n' },
+	{ "iac",	1, 0, 'i' },
 	{ "flush",	0, 0, 'f' },
 	{ 0, 0, 0, 0 }
 };
@@ -331,14 +328,16 @@ static char *inq_help =
 	"Usage:\n"
 	"\tinq [--length=N] maximum inquiry duration in 1.28 s units\n"
 	"\t    [--numrsp=N] specify maximum number of inquiry responses\n"
+	"\t    [--iac=lap]  specify the inquiry access code\n"
 	"\t    [--flush]    flush the inquiry cache\n";
 
 static void cmd_inq(int dev_id, int argc, char **argv)
 {
-	int num_rsp, length, flags;
 	inquiry_info *info = NULL;
+	uint8_t lap[3] = { 0x33, 0x8b, 0x9e };
+	int num_rsp, length, flags;
 	char addr[18];
-	int i, opt;
+	int i, l, opt;
 
 	length  = 8;	/* ~10 seconds */
 	num_rsp = 0;
@@ -354,6 +353,21 @@ static void cmd_inq(int dev_id, int argc, char **argv)
 			num_rsp = atoi(optarg);
 			break;
 
+		case 'i':
+			l = strtoul(optarg, 0, 16);
+			if (!strcasecmp(optarg, "giac")) {
+				l = 0x9e8b33;
+			} else if (!strcasecmp(optarg, "liac")) {
+				l = 0x9e8b00;
+			} if (l < 0x9e8b00 || l > 0x9e8b3f) {
+				printf("Invalid access code 0x%x\n", l);
+				exit(1);
+			}
+			lap[0] = (l & 0xff);
+			lap[1] = (l >> 8) & 0xff;
+			lap[2] = (l >> 16) & 0xff;
+			break;
+
 		case 'f':
 			flags |= IREQ_CACHE_FLUSH;
 			break;
@@ -365,7 +379,7 @@ static void cmd_inq(int dev_id, int argc, char **argv)
 	}
 
 	printf("Inquiring ...\n");
-	num_rsp = hci_inquiry(dev_id, length, num_rsp, NULL, &info, flags);
+	num_rsp = hci_inquiry(dev_id, length, num_rsp, lap, &info, flags);
 	if (num_rsp < 0) {
 		perror("Inquiry failed.");
 		exit(1);
@@ -389,22 +403,24 @@ static struct option scan_options[] = {
 	{ "help",	0, 0, 'h' },
 	{ "length",	1, 0, 'l' },
 	{ "numrsp",	1, 0, 'n' },
+	{ "iac",	1, 0, 'i' },
 	{ "flush",	0, 0, 'f' },
-	{ "class",	0, 0, 'c' },
-	{ "info",	0, 0, 'i' },
-	{ "oui",	0, 0, 'o' },
-	{ "all",	0, 0, 'a' },
-	{ "ext",	0, 0, 'a' },
+	{ "class",	0, 0, 'C' },
+	{ "info",	0, 0, 'I' },
+	{ "oui",	0, 0, 'O' },
+	{ "all",	0, 0, 'A' },
+	{ "ext",	0, 0, 'A' },
 	{ 0, 0, 0, 0 }
 };
 
 static char *scan_help =
 	"Usage:\n"
-	"\tscan [--length=N] [--numrsp=N] [--flush] [--class] [--info] [--oui]\n";
+	"\tscan [--length=N] [--numrsp=N] [--iac=lap] [--flush] [--class] [--info] [--oui]\n";
 
 static void cmd_scan(int dev_id, int argc, char **argv)
 {
 	inquiry_info *info = NULL;
+	uint8_t lap[3] = { 0x33, 0x8b, 0x9e };
 	int num_rsp, length, flags;
 	uint8_t cls[3], features[8];
 	uint16_t handle;
@@ -413,7 +429,7 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 	struct hci_dev_info di;
 	struct hci_conn_info_req *cr;
 	int extcls = 0, extinf = 0, extoui = 0;
-	int i, n, opt, dd, cc, nc;
+	int i, n, l, opt, dd, cc, nc;
 
 	length  = 8;	/* ~10 seconds */
 	num_rsp = 0;
@@ -429,23 +445,38 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 			num_rsp = atoi(optarg);
 			break;
 
+		case 'i':
+			l = strtoul(optarg, 0, 16);
+			if (!strcasecmp(optarg, "giac")) {
+				l = 0x9e8b33;
+			} else if (!strcasecmp(optarg, "liac")) {
+				l = 0x9e8b00;
+			} else if (l < 0x9e8b00 || l > 0x9e8b3f) {
+				printf("Invalid access code 0x%x\n", l);
+				exit(1);
+			}
+			lap[0] = (l & 0xff);
+			lap[1] = (l >> 8) & 0xff;
+			lap[2] = (l >> 16) & 0xff;
+			break;
+
 		case 'f':
 			flags |= IREQ_CACHE_FLUSH;
 			break;
 
-		case 'c':
+		case 'C':
 			extcls = 1;
 			break;
 
-		case 'i':
+		case 'I':
 			extinf = 1;
 			break;
 
-		case 'o':
+		case 'O':
 			extoui = 1;
 			break;
 
-		case 'a':
+		case 'A':
 			extcls = 1;
 			extinf = 1;
 			extoui = 1;
@@ -471,7 +502,7 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 	}
 
 	printf("Scanning ...\n");
-	num_rsp = hci_inquiry(dev_id, length, num_rsp, NULL, &info, flags);
+	num_rsp = hci_inquiry(dev_id, length, num_rsp, lap, &info, flags);
 	if (num_rsp < 0) {
 		perror("Inquiry failed");
 		exit(1);
